'use strict';

let startButton = document.getElementById('startButton');
let callButton = document.getElementById('callButton');
let hangupButton = document.getElementById('hangupButton');
callButton.disabled = true;
hangupButton.disabled = true;
startButton.onclick = start;
callButton.onclick = call;
hangupButton.onclick = hangup;


let video1 = document.querySelector('video#video1');

let clients = [
    {video: document.querySelector('video#video2')},
    {video: document.querySelector('video#video3')},
    {video: document.querySelector('video#video4')},
    {video: document.querySelector('video#video5')},
    {video: document.querySelector('video#video6')},
    {video: document.querySelector('video#video7')},
    {video: document.querySelector('video#video8')},
    {video: document.querySelector('video#video9')},
    {video: document.querySelector('video#video10')},
    {video: document.querySelector('video#video11')}
];

let offerOptions = {
    offerToReceiveAudio: 1,
    offerToReceiveVideo: 1
};

function getStream() {
    trace('Received local stream');
    if (video1.captureStream) {
        window.localStream = video1.captureStream();
    } else if (video1.mozCaptureStream) {
        window.localStream = video1.mozCaptureStream();
    } else {
        return;
    }
    callButton.disabled = false;
}

function start() {
    trace('Requesting local stream');
    startButton.disabled = true;
    video1.play();
    getStream();
}

function call() {
    callButton.disabled = true;
    hangupButton.disabled = false;
    trace('Starting calls');
    let audioTracks = window.localStream.getAudioTracks();
    let videoTracks = window.localStream.getVideoTracks();
    if (audioTracks.length > 0) {
        // trace('Using audio device: ' + audioTracks[0].label);
    }
    if (videoTracks.length > 0) {
        // trace('Using video device: ' + videoTracks[0].label);
    }

    let servers = null;
    for (let i = 0; i < clients.length; i++){
        let vEl = clients[i].video;
        clients[i].local = new RTCPeerConnection(servers);
        clients[i].remote = new RTCPeerConnection(servers);
        let client = clients[i];
        client.remote.ontrack = function(e){
            gotRemoteStream(vEl, e);
        };
        client.local.onicecandidate = function(e){
            iceCallbackLocal(clients[i], e);
        };
        client.remote.onicecandidate = function(e){
            iceCallbackRemote(clients[i], e);
        };
        trace(client.video.id + ': created local and remote peer connection objects');
        window.localStream.getTracks().forEach(
            function (track) {
                client.local.addTrack(
                    track,
                    window.localStream
                );
            }
        );
        trace('Adding local stream to ' + clients[i].video.id + 'local');
        client.local.createOffer(
            offerOptions
        ).then(
            function(descr){gotDescriptionLocal(client, descr);},
            onCreateSessionDescriptionError
        );
    }
}

function onCreateSessionDescriptionError(error) {
    trace('Failed to create session description: ' + error.toString());
}


function gotDescriptionLocal(client, desc){
    client.local.setLocalDescription(desc);
    trace('Offer from' + client.video.id + 'local \n'); // + desc.sdp
    client.remote.setRemoteDescription(desc);
    client.remote.createAnswer().then(
        function(descr){gotDescriptionRemote(client, descr);},
        onCreateSessionDescriptionError
    );
}

function gotDescriptionRemote(client, desc){
    client.remote.setLocalDescription(desc);
    trace('Answer from ' + client.video.id +'remote\n');  // + desc.sdp
    client.local.setRemoteDescription(desc);
}


function hangup() {
    trace('Ending calls');
    for (let i = 0; i < clients.length; i++){
        let client = clients[i];
        client.local.close();
        client.remote.close();
        client.local = null;
        client.remote = null;
    }
    hangupButton.disabled = true;
    callButton.disabled = false;
}

function gotRemoteStream(vEl, e){
    if (vEl.srcObject !== e.streams[0]){
        vEl.srcObject = e.streams[0];
        trace(vEl.id + ': received remote stream');
    }
}



function iceCallbackLocal(client, event){
    handleCandidate(event.candidate, client.remote, client.video.id, 'local');
}

function iceCallbackRemote(client, event){
    handleCandidate(event.candidate, client.local, client.video.id, 'remote');
}

function handleCandidate(candidate, dest, prefix, type) {
    dest.addIceCandidate(candidate)
        .then(
            onAddIceCandidateSuccess,
            onAddIceCandidateError
        );
    // trace(prefix + 'New ' + type + ' ICE candidate: ' +
    //     (candidate ? candidate.candidate : '(null)'));
}

function onAddIceCandidateSuccess() {
    // trace('AddIceCandidate success.');
}

function onAddIceCandidateError(error) {
    trace('Failed to add ICE candidate: ' + error.toString());
}
